@@ -18,7 +18,7 @@
  * Copyright (C) 2001 Ramalingam Saravanan. All Rights Reserved.
  *
  * Contributor(s):
- * Patrick Brunschwig <patrick@mozilla-enigmail.org>
+ * Patrick Brunschwig <patrick@enigmail.net>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -33,6 +33,8 @@
  * the terms of any one of the MPL, the GPL or the LGPL.
  * ***** END LICENSE BLOCK ***** */
 
+'use strict';
+
 Components.utils.import("resource://enigmail/enigmailCommon.jsm");
 Components.utils.import("resource://enigmail/commonFuncs.jsm");
 Components.utils.import("resource://enigmail/mimeVerify.jsm");
@@ -65,15 +67,28 @@ Enigmail.hdrView = {
     this.statusBar = document.getElementById("enigmail-status-bar");
     this.enigmailBox = document.getElementById("enigmailBox");
 
+    var addrPopup = document.getElementById("emailAddressPopup");
+    if (addrPopup) {
+      var attr = addrPopup.getAttribute("onpopupshowing");
+      attr = "EnigmailFuncs.collapseAdvanced(this, 'hidden'); "+attr;
+      addrPopup.setAttribute("onpopupshowing", attr);
+    }
   },
 
+
   statusBarHide: function ()
   {
     try {
       this.statusBar.removeAttribute("signed");
       this.statusBar.removeAttribute("encrypted");
-      this.enigmailBox.setAttribute("collapsed", "true")
+      this.enigmailBox.setAttribute("collapsed", "true");
       Enigmail.msg.setAttachmentReveal(null);
+      if (Enigmail.msg.securityInfo) {
+        Enigmail.msg.securityInfo.statusFlags = 0;
+        Enigmail.msg.securityInfo.msgSigned = 0;
+        Enigmail.msg.securityInfo.msgEncrypted = 0;
+      }
+
     }
     catch (ex) {}
   },
@@ -104,7 +119,12 @@ Enigmail.hdrView = {
   },
 
 
-  updateHdrIcons: function (exitCode, statusFlags, keyId, userId, sigDetails, errorMsg, blockSeparation)
+  setStatusText: function(txt) {
+    let s = document.getElementById("enigmailStatusText");
+    s.firstChild.data = txt;
+  },
+
+  updateHdrIcons: function (exitCode, statusFlags, keyId, userId, sigDetails, errorMsg, blockSeparation, encToDetails, xtraStatus)
   {
     EnigmailCommon.DEBUG_LOG("enigmailMsgHdrViewOverlay.js: this.updateHdrIcons: exitCode="+exitCode+", statusFlags="+statusFlags+", keyId="+keyId+", userId="+userId+", "+errorMsg+"\n");
 
@@ -113,7 +133,6 @@ Enigmail.hdrView = {
     this.statusBar = document.getElementById("enigmail-status-bar");
     this.enigmailBox = document.getElementById("enigmailBox");
 
-
     if (gFolderDisplay.selectedMessageUris.length > 0) {
       this.lastEncryptedMsgKey = gFolderDisplay.selectedMessageUris[0];
     }
@@ -155,7 +174,6 @@ Enigmail.hdrView = {
       }
     }
 
-
     if (errorLines && (errorLines.length > 22) ) {
       // Retain only first twenty lines and last two lines of error message
       var lastLines = errorLines[errorLines.length-2] + "\n" +
@@ -191,29 +209,30 @@ Enigmail.hdrView = {
     }
 
     var msgSigned = (statusFlags & (nsIEnigmail.BAD_SIGNATURE |
-              nsIEnigmail.GOOD_SIGNATURE |
-              nsIEnigmail.EXPIRED_KEY_SIGNATURE |
-              nsIEnigmail.EXPIRED_SIGNATURE |
-              nsIEnigmail.UNVERIFIED_SIGNATURE |
-              nsIEnigmail.REVOKED_KEY |
-              nsIEnigmail.EXPIRED_KEY_SIGNATURE |
-              nsIEnigmail.EXPIRED_SIGNATURE));
+                                    nsIEnigmail.GOOD_SIGNATURE |
+                                    nsIEnigmail.EXPIRED_KEY_SIGNATURE |
+                                    nsIEnigmail.EXPIRED_SIGNATURE |
+                                    nsIEnigmail.UNVERIFIED_SIGNATURE |
+                                    nsIEnigmail.REVOKED_KEY |
+                                    nsIEnigmail.EXPIRED_KEY_SIGNATURE |
+                                    nsIEnigmail.EXPIRED_SIGNATURE));
     var msgEncrypted = (statusFlags & (nsIEnigmail.DECRYPTION_OKAY |
-              nsIEnigmail.DECRYPTION_INCOMPLETE |
-              nsIEnigmail.DECRYPTION_FAILED));
+                                       nsIEnigmail.DECRYPTION_INCOMPLETE |
+                                       nsIEnigmail.DECRYPTION_FAILED));
 
     if (msgSigned && (statusFlags & nsIEnigmail.IMPORTED_KEY)) {
       statusFlags &= (~nsIEnigmail.IMPORTED_KEY);
     }
 
     if (((!(statusFlags & (nsIEnigmail.DECRYPTION_INCOMPLETE |
-              nsIEnigmail.DECRYPTION_FAILED |
-              nsIEnigmail.UNVERIFIED_SIGNATURE |
-              nsIEnigmail.BAD_SIGNATURE))) ||
+                           nsIEnigmail.DECRYPTION_FAILED |
+                           nsIEnigmail.UNVERIFIED_SIGNATURE |
+                           nsIEnigmail.BAD_SIGNATURE)))
+         ||
          (statusFlags & nsIEnigmail.DISPLAY_MESSAGE) &&
           !(statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE)) &&
             !(statusFlags & nsIEnigmail.IMPORTED_KEY)) {
-      // Normal exit / display message
+      // normal exit / display message
       statusLine = errorMsg;
       statusInfo = statusLine;
 
@@ -237,78 +256,92 @@ Enigmail.hdrView = {
       }
       fullStatusInfo = statusInfo;
 
-    } else {
-      if (keyId) {
-        statusInfo = EnigmailCommon.getString("keyNeeded", [ keyId ]);
-
-        if (statusFlags & nsIEnigmail.INLINE_KEY) {
-          statusLine = statusInfo + EnigmailCommon.getString("clickDecrypt");
-        } else {
-          statusLine = statusInfo + EnigmailCommon.getString("clickPen");
-        }
-
-        statusInfo = EnigmailCommon.getString("unverifiedSig");
-        statusLine = statusInfo + EnigmailCommon.getString("clickPen");
-        statusInfo += "\n\n" + errorMsg;
-
-      } else if (statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE) {
-        statusInfo = EnigmailCommon.getString("unverifiedSig");
-        statusLine = statusInfo + EnigmailCommon.getString("clickQueryPenDetails");
-        statusInfo += "\n\n" + errorMsg;
-
-      } else if (statusFlags & (nsIEnigmail.BAD_SIGNATURE |
-                                nsIEnigmail.UNVERIFIED_SIGNATURE |
-                                nsIEnigmail.EXPIRED_SIGNATURE |
-                                nsIEnigmail.EXPIRED_KEY_SIGNATURE)) {
-        statusInfo = EnigmailCommon.getString("failedSig");
-        statusLine = statusInfo + EnigmailCommon.getString("clickPenDetails");
-        statusInfo += "\n\n" + errorMsg;
-
-      } else if (statusFlags & nsIEnigmail.DECRYPTION_INCOMPLETE) {
-        statusInfo = EnigmailCommon.getString("incompleteDecrypt");
-        statusLine = statusInfo + EnigmailCommon.getString("clickKey");
-        statusInfo += "\n\n" + errorMsg;
-
-      } else if (statusFlags & nsIEnigmail.DECRYPTION_FAILED) {
+    }
+    else {
+      // no normal exit / don't display message
+      // - process failed decryptions first because they imply bad signature handling
+      if (statusFlags & nsIEnigmail.DECRYPTION_FAILED) {
         if (statusFlags & nsIEnigmail.NO_SECKEY) {
           statusInfo = EnigmailCommon.getString("needKey");
         } else {
           statusInfo = EnigmailCommon.getString("failedDecrypt");
         }
-
-        statusLine = statusInfo + EnigmailCommon.getString("clickKeyDetails");
-        statusInfo += "\n\n" + errorMsg;
-
-      } else if (statusFlags & nsIEnigmail.BAD_PASSPHRASE) {
+        statusLine = statusInfo + EnigmailCommon.getString("clickDetailsButton");
+      }
+      else if (statusFlags & nsIEnigmail.BAD_PASSPHRASE) {
         statusInfo = EnigmailCommon.getString("badPhrase");
         statusLine = statusInfo + EnigmailCommon.getString("clickDecryptRetry");
-        statusInfo += "\n\n" + errorMsg;
-
-      } else if (statusFlags & nsIEnigmail.IMPORTED_KEY) {
+      }
+      else if (statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE) {
+        statusInfo = EnigmailCommon.getString("unverifiedSig");
+        if (keyId) {
+          statusLine = statusInfo + EnigmailCommon.getString("clickImportButton");
+        }
+        else {
+          statusLine = statusInfo + EnigmailCommon.getString("keyTypeUnsupported");
+        }
+      }
+      else if (statusFlags & (nsIEnigmail.BAD_SIGNATURE |
+                              nsIEnigmail.EXPIRED_SIGNATURE |
+                              nsIEnigmail.EXPIRED_KEY_SIGNATURE)) {
+        statusInfo = EnigmailCommon.getString("failedSig");
+        statusLine = statusInfo + EnigmailCommon.getString("clickDetailsButton");
+      }
+      else if (statusFlags & nsIEnigmail.DECRYPTION_INCOMPLETE) {
+        statusInfo = EnigmailCommon.getString("incompleteDecrypt");
+        statusLine = statusInfo + EnigmailCommon.getString("clickDetailsButton");
+      }
+      else if (statusFlags & nsIEnigmail.IMPORTED_KEY) {
         statusLine = "";
         statusInfo = "";
         EnigmailCommon.alert(window, errorMsg);
-
-      } else {
+      }
+      else {
         statusInfo = EnigmailCommon.getString("failedDecryptVerify");
         statusLine = statusInfo + EnigmailCommon.getString("viewInfo");
-        statusInfo += "\n\n" + errorMsg;
       }
+      // add key infos if available
+      if (keyId) {
+        var si = EnigmailCommon.getString("unverifiedSig");  // "Unverified signature"
+        if (statusInfo == "") {
+          statusInfo += si;
+          statusLine = si + EnigmailCommon.getString("clickDetailsButton");
+        }
+        //if (statusFlags & nsIEnigmail.INLINE_KEY) {
+        //  statusLine = statusInfo + EnigmailCommon.getString("clickDecrypt");
+        //} else {
+        //  statusLine = statusInfo + EnigmailCommon.getString("clickPen");
+        //}
+        if (statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE) {
+          statusInfo += "\n" + EnigmailCommon.getString("keyNeeded", [ keyId ]);  // "public key ... needed"
+        }
+        else {
+          statusInfo += "\n" + EnigmailCommon.getString("keyUsed", [ keyId ]);  // "public key ... used"
+        }
+      }
+      statusInfo += "\n\n" + errorMsg;
     }
 
     if (statusFlags & nsIEnigmail.DECRYPTION_OKAY ||
         (this.statusBar.getAttribute("encrypted")=="ok")) {
-      if (!statusInfo) {
-        statusInfo = EnigmailCommon.getString("decryptedMsg");
+      var statusMsg;
+      if (xtraStatus && xtraStatus == "buggyMailFormat") {
+        statusMsg = EnigmailCommon.getString("decryptedMsgWithFormatError");
       }
       else {
-        statusInfo = EnigmailCommon.getString("decryptedMsg")+"\n"+statusInfo;
+        statusMsg = EnigmailCommon.getString("decryptedMsg");
+      }
+      if (!statusInfo) {
+        statusInfo = statusMsg;
+      }
+      else {
+        statusInfo = statusMsg + "\n" + statusInfo;
       }
       if (!statusLine) {
-        statusLine=statusInfo;
+        statusLine = statusInfo;
       }
       else {
-        statusLine=EnigmailCommon.getString("decryptedMsg")+"; "+statusLine;
+        statusLine = statusMsg + "; " + statusLine;
       }
     }
 
@@ -316,19 +349,45 @@ Enigmail.hdrView = {
       if (statusFlags & nsIEnigmail.PARTIALLY_PGP) {
         if (msgSigned && msgEncrypted) {
           statusLine = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgSignedAndEnc") ]);
-          statusLine += EnigmailCommon.getString("clickPenKeyDetails");
+          statusLine += EnigmailCommon.getString("clickDetailsButton");
+          statusInfo = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgSigned") ])
+                        + "\n" + statusInfo;
         }
         else if (msgEncrypted) {
           statusLine = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgEncrypted") ]);
-          statusLine += EnigmailCommon.getString("clickQueryKeyDetails");
+          statusLine += EnigmailCommon.getString("clickDetailsButton");
+          statusInfo = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgEncrypted") ])
+                        + "\n" + statusInfo;
         }
         else if (msgSigned) {
-          statusLine = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgSigned") ]);
-          statusLine += EnigmailCommon.getString("clickQueryPenDetails");
+          if (statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE) {
+            statusLine = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgSignedUnkownKey") ]);
+            if (keyId) {
+              statusLine += EnigmailCommon.getString("clickImportButton");
+            }
+            else {
+              statusLine += EnigmailCommon.getString("keyTypeUnsupported");
+            }
+          }
+          else {
+            statusLine = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgSigned") ]);
+            statusLine += EnigmailCommon.getString("clickDetailsButton");
+          }
+          statusInfo = EnigmailCommon.getString("msgPart", [ EnigmailCommon.getString("msgSigned") ])
+                        + "\n" + statusInfo;
         }
       }
     }
 
+    // if we have parsed ENC_TO entries, add them as status info
+    if (encToDetails && encToDetails.length > 0) {
+      statusInfo += "\n\n" + EnigmailCommon.getString("encryptKeysNote", [ encToDetails ]);
+    }
+
+    if (! statusLine) {
+      return;
+    }
+
     Enigmail.msg.securityInfo = { statusFlags: statusFlags,
                           keyId: keyId,
                           userId: userId,
@@ -355,11 +414,20 @@ Enigmail.hdrView = {
     }
 
     if (statusLine) {
-      statusText.value = statusLine +" ";
+      this.setStatusText(statusLine +" ");
       this.enigmailBox.removeAttribute("collapsed");
       this.displayExtendedStatus(true);
+
+      if (Enigmail.msg.securityInfo.keyId &&
+          (Enigmail.msg.securityInfo.statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE) ) {
+        document.getElementById("enigmail_importKey").removeAttribute("hidden");
+      }
+      else {
+        document.getElementById("enigmail_importKey").setAttribute("hidden", "true");
+      }
+
     } else {
-      statusText.value = "";
+      this.setStatusText("");
       this.enigmailBox.setAttribute("collapsed", "true");
       this.displayExtendedStatus(false);
     }
@@ -373,7 +441,8 @@ Enigmail.hdrView = {
       gSignedUINode.collapsed = false;
       gEncryptedUINode.collapsed = false;
 
-      if (statusFlags & nsIEnigmail.BAD_SIGNATURE) {
+      if ((statusFlags & nsIEnigmail.BAD_SIGNATURE) &&
+          !(statusFlags & nsIEnigmail.GOOD_SIGNATURE)){
         // Display untrusted/bad signature icon
         gSignedUINode.setAttribute("signed", "notok");
         this.enigmailBox.setAttribute("class", "expandedEnigmailBox enigmailHeaderBoxLabelSignatureNotOk");
@@ -426,8 +495,15 @@ Enigmail.hdrView = {
         this.statusBar.setAttribute("encrypted", "notok");
         this.enigmailBox.setAttribute("class", "expandedEnigmailBox enigmailHeaderBoxLabelSignatureNotOk");
       }
+
+      // special handling after trying to fix buggy mail format (see buggyExchangeEmailContent in code)
+      if (xtraStatus && xtraStatus == "buggyMailFormat") {
+        this.enigmailBox.setAttribute("class", "expandedEnigmailBox enigmailHeaderBoxLabelBuggyMailFormat");
+      }
+
       this.updateMsgDb();
 
+
     } catch (ex) {}
   },
 
@@ -436,58 +512,71 @@ Enigmail.hdrView = {
 
     const nsIEnigmail = Components.interfaces.nsIEnigmail;
 
-    if (Enigmail.msg.securityInfo) {
-      if (Enigmail.msg.securityInfo.keyId &&
-          (Enigmail.msg.securityInfo.statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE) ) {
-        document.getElementById("enigmail_importKey").removeAttribute("hidden");
-      }
-      else {
-        document.getElementById("enigmail_importKey").setAttribute("hidden", "true");
-      }
-
-      if ( (Enigmail.msg.securityInfo.statusFlags & nsIEnigmail.NODATA) &&
-           (Enigmail.msg.securityInfo.statusFlags &
-             (nsIEnigmail.PGP_MIME_SIGNED | nsIEnigmail.PGP_MIME_ENCRYPTED)) ) {
-        document.getElementById("enigmail_reloadMessage").removeAttribute("hidden");
-      }
-      else {
-        document.getElementById("enigmail_reloadMessage").setAttribute("hidden", "true");
-      }
-    }
-
-    var optList = ["pgpSecurityInfo", "copySecurityInfo"];
-    for (var j=0; j<optList.length; j++) {
-      var menuElement = document.getElementById("enigmail_"+optList[j]);
+    try {
       if (Enigmail.msg.securityInfo) {
-        menuElement.removeAttribute("disabled");
+        if ( (Enigmail.msg.securityInfo.statusFlags & nsIEnigmail.NODATA) &&
+             (Enigmail.msg.securityInfo.statusFlags &
+               (nsIEnigmail.PGP_MIME_SIGNED | nsIEnigmail.PGP_MIME_ENCRYPTED)) ) {
+          document.getElementById("enigmail_reloadMessage").removeAttribute("hidden");
+        }
+        else {
+          document.getElementById("enigmail_reloadMessage").setAttribute("hidden", "true");
+        }
       }
-      else {
-        menuElement.setAttribute("disabled", "true");
-      }
-    }
 
-    this.setSenderStatus("signSenderKey", "editSenderKeyTrust" , "showPhoto", "dispKeyDetails");
+      var optList = ["pgpSecurityInfo", "copySecurityInfo"];
+      for (var j=0; j<optList.length; j++) {
+        var menuElement = document.getElementById("enigmail_"+optList[j]);
+        if (Enigmail.msg.securityInfo) {
+          menuElement.removeAttribute("disabled");
+        }
+        else {
+          menuElement.setAttribute("disabled", "true");
+        }
+      }
+
+      this.setSenderStatus("signSenderKey", "editSenderKeyTrust" , "showPhoto", "dispKeyDetails");
+    }
+    catch(ex) {
+      EnigmailCommon.ERROR_LOG("error on displaying Security menu:\n"+ex.toString()+"\n")
+    }
   },
 
 
   updateSendersKeyMenu: function ()
   {
-    this.setSenderStatus("keyMgmtSignKey", "keyMgmtKeyTrust", "keyMgmtShowPhoto", "keyMgmtDispKeyDetails");
+    this.setSenderStatus("keyMgmtSignKey",
+                         "keyMgmtKeyTrust",
+                         "keyMgmtShowPhoto",
+                         "keyMgmtDispKeyDetails",
+                         "importpublickey");
   },
 
 
-  setSenderStatus: function (elemSign, elemTrust, elemPhoto, elemKeyProps)
+  setSenderStatus: function (elemSign, elemTrust, elemPhoto, elemKeyProps, elemImportKey)
   {
+
+    function setElemStatus(elemName, disabledValue) {
+      document.getElementById("enigmail_"+elemName).setAttribute("disabled", !disabledValue);
+
+      let secondElem = document.getElementById("enigmail_"+elemName+"2");
+      if (secondElem) secondElem.setAttribute("disabled", !disabledValue);
+    }
+
     const nsIEnigmail = Components.interfaces.nsIEnigmail;
 
     var photo=false;
     var sign=false;
     var trust=false;
+    var unknown = false;
+    var signedMsg = false;
+
     if (Enigmail.msg.securityInfo) {
       if (Enigmail.msg.securityInfo.statusFlags & nsIEnigmail.PHOTO_AVAILABLE) {
         photo=true;
       }
       if (Enigmail.msg.securityInfo.msgSigned ) {
+        signedMsg = true;
         if (!(Enigmail.msg.securityInfo.statusFlags &
              (nsIEnigmail.REVOKED_KEY | nsIEnigmail.EXPIRED_KEY_SIGNATURE | nsIEnigmail.UNVERIFIED_SIGNATURE))) {
           sign=true;
@@ -495,29 +584,41 @@ Enigmail.hdrView = {
         if (!(Enigmail.msg.securityInfo.statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE)) {
           trust=true;
         }
+
+        if (Enigmail.msg.securityInfo.statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE) {
+          unknown = true;
+        }
       }
     }
 
-    if (elemTrust)
-      document.getElementById("enigmail_"+elemTrust).setAttribute("disabled", !trust);
-    if (elemSign)
-      document.getElementById("enigmail_"+elemSign).setAttribute("disabled", !sign);
-    if (elemPhoto)
-      document.getElementById("enigmail_"+elemPhoto).setAttribute("disabled", !photo);
-    if (elemKeyProps)
-      document.getElementById("enigmail_"+elemKeyProps).setAttribute("disabled", !sign);
+    if (elemTrust) setElemStatus(elemTrust, trust);
+    if (elemSign) setElemStatus(elemSign, sign);
+    if (elemPhoto) setElemStatus(elemPhoto, photo);
+    if (elemKeyProps) setElemStatus(elemKeyProps, (signedMsg && !unknown));
+    if (elemImportKey) setElemStatus(elemImportKey, unknown);
+  },
 
+  editKeyExpiry: function ()
+  {
+    EnigmailFuncs.editKeyExpiry(window, [Enigmail.msg.securityInfo.userId], [Enigmail.msg.securityInfo.keyId]);
+    gDBView.reloadMessageWithAllParts();
   },
 
   editKeyTrust: function ()
   {
-    EnigmailFuncs.editKeyTrust(window, [Enigmail.msg.securityInfo.userId], [Enigmail.msg.securityInfo.keyId]);
+    let enigmailSvc = EnigmailCommon.getService();
+    let keyId = enigmailSvc.getPubKeyIdForSubkey(Enigmail.msg.securityInfo.keyId);
+
+    EnigmailFuncs.editKeyTrust(window, [Enigmail.msg.securityInfo.userId], [keyId]);
     gDBView.reloadMessageWithAllParts();
   },
 
   signKey: function ()
   {
-    EnigmailFuncs.signKey(window, Enigmail.msg.securityInfo.userId, Enigmail.msg.securityInfo.keyId, null)
+    let enigmailSvc = EnigmailCommon.getService();
+    let keyId = enigmailSvc.getPubKeyIdForSubkey(Enigmail.msg.securityInfo.keyId);
+
+    EnigmailFuncs.signKey(window, Enigmail.msg.securityInfo.userId, keyId, null);
     gDBView.reloadMessageWithAllParts();
   },
 
@@ -535,10 +636,10 @@ Enigmail.hdrView = {
         try {
 
           Enigmail.hdrView.statusBarHide();
+
           EnigmailVerify.setMsgWindow(msgWindow, Enigmail.msg.getCurrentMsgUriSpec());
 
-          var statusText = document.getElementById("enigmailStatusText");
-          if (statusText) statusText.value="";
+          Enigmail.hdrView.setStatusText("");
 
           this.enigmailBox.setAttribute("class", "expandedEnigmailBox enigmailHeaderBoxLabelSignatureOk");
 
@@ -568,12 +669,12 @@ Enigmail.hdrView = {
         EnigmailCommon.DEBUG_LOG("enigmailMsgHdrViewOverlay.js: _listener_onEndHeaders\n");
         try {
           Enigmail.hdrView.statusBarHide();
-          var statusText = document.getElementById("enigmailStatusText");
 
           this.enigmailBox.setAttribute("class", "expandedEnigmailBox enigmailHeaderBoxLabelSignatureOk");
         }
         catch (ex) {}
       },
+
       beforeStartHeaders: function _listener_beforeStartHeaders ()
       {
         return true;
@@ -598,7 +699,7 @@ Enigmail.hdrView = {
   {
     if (Enigmail.msg.securityInfo) {
       var clipHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].createInstance(Components.interfaces.nsIClipboardHelper);
-      clipHelper.copyString(Enigmail.msg.securityInfo.fullStatusInfo);
+      clipHelper.copyString(Enigmail.msg.securityInfo.statusInfo);
     }
 
   },
@@ -607,7 +708,10 @@ Enigmail.hdrView = {
   {
     if (! Enigmail.msg.securityInfo) return;
 
-    EnigmailFuncs.showPhoto(window, Enigmail.msg.securityInfo.keyId, Enigmail.msg.securityInfo.userId);
+    let enigmailSvc = EnigmailCommon.getService();
+    let keyId = enigmailSvc.getPubKeyIdForSubkey(Enigmail.msg.securityInfo.keyId);
+
+    EnigmailFuncs.showPhoto(window, keyId, Enigmail.msg.securityInfo.userId);
   },
 
 
@@ -615,7 +719,10 @@ Enigmail.hdrView = {
   {
     if (! Enigmail.msg.securityInfo) return;
 
-    EnigmailFuncs.openKeyDetails(window, Enigmail.msg.securityInfo.keyId, false);
+    let enigmailSvc = EnigmailCommon.getService();
+    let keyId = enigmailSvc.getPubKeyIdForSubkey(Enigmail.msg.securityInfo.keyId);
+
+    EnigmailFuncs.openKeyDetails(window, keyId, false);
   },
 
   createRuleFromAddress: function (emailAddressNode)
@@ -652,9 +759,9 @@ Enigmail.hdrView = {
 
   },
 
-  msgHdrViewUnide: function ()
+  msgHdrViewUnhide: function (event)
   {
-    EnigmailCommon.DEBUG_LOG("enigmailMsgHdrViewOverlay.js: this.msgHdrViewUnide\n");
+    EnigmailCommon.DEBUG_LOG("enigmailMsgHdrViewOverlay.js: this.msgHdrViewUnhide:\n");
 
     if (Enigmail.msg.securityInfo.statusFlags != 0) {
       this.enigmailBox.removeAttribute("collapsed");
@@ -748,12 +855,12 @@ Enigmail.hdrView = {
         verifyMenu.setAttribute('disabled', true);
         if (typeof(selectedAttachments[0].displayName) == "undefined") {
           if (! selectedAttachments[0].name) {
-            selectedAttachments[0].name="message.pgp"
+            selectedAttachments[0].name="message.pgp";
           }
         }
         else
           if (! selectedAttachments[0].displayName) {
-            selectedAttachments[0].displayName="message.pgp"
+            selectedAttachments[0].displayName="message.pgp";
           }
       }
       else {
@@ -816,13 +923,14 @@ Enigmail.hdrView = {
       }
     }
   }
+
 };
 
-window.addEventListener("load", Enigmail.hdrView.hdrViewLoad.bind(Enigmail.hdrView), false)
+window.addEventListener("load", Enigmail.hdrView.hdrViewLoad.bind(Enigmail.hdrView), false);
 addEventListener('messagepane-loaded', Enigmail.hdrView.msgHdrViewLoad.bind(Enigmail.hdrView), true);
 addEventListener('messagepane-unloaded', Enigmail.hdrView.hdrViewUnload.bind(Enigmail.hdrView), true);
 addEventListener('messagepane-hide', Enigmail.hdrView.msgHdrViewHide.bind(Enigmail.hdrView), true);
-addEventListener('messagepane-unhide', Enigmail.hdrView.msgHdrViewUnide.bind(Enigmail.hdrView), true);
+addEventListener('messagepane-unhide', Enigmail.hdrView.msgHdrViewUnhide.bind(Enigmail.hdrView), true);
 
 ////////////////////////////////////////////////////////////////////////////////
 // THE FOLLOWING OVERRIDES CODE IN msgHdrViewOverlay.js
@@ -844,27 +952,6 @@ function CanDetachAttachments()
   return canDetach && Enigmail.hdrView.enigCanDetachAttachments();
 }
 
-// Distinction between createNewAttachmentInfo and AttachmentInfo
-// due to renamed function in MsgHdrView.js in TB trunk code.
-// Can be removed in later versions of Enigmail.
-
-try
-{
-     createNewAttachmentInfo.prototype.origOpenAttachment = createNewAttachmentInfo.prototype.openAttachment;
-     createNewAttachmentInfo.prototype.openAttachment = function ()
-     {
-       this.origOpenAttachment();
-     }
-}
-catch (ex)
-{
-    AttachmentInfo.prototype.origOpenAttachment = AttachmentInfo.prototype.openAttachment;
-    AttachmentInfo.prototype.openAttachment = function ()
-    {
-      this.origOpenAttachment();
-    }
-}
-
 
 ////////////////////////////////////////////////////////////////////////////////
 // THE FOLLOWING EXTENDS CODE IN msgHdrViewOverlay.js
@@ -900,17 +987,40 @@ if (messageHeaderSink) {
         throw Components.results.NS_NOINTERFACE;
       },
 
-      updateSecurityStatus: function (uriSpec, exitCode, statusFlags, keyId, userId, sigDetails, errorMsg, blockSeparation, uri)
+      updateSecurityStatus: function (unusedUriSpec, exitCode, statusFlags, keyId, userId, sigDetails, errorMsg, blockSeparation, uri, encToDetails)
       {
-        // uri is not used here; added for compatibility to other addons
-        EnigmailCommon.DEBUG_LOG("enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.updateSecurityStatus: uriSpec="+uriSpec+"\n");
+        // unusedUriSpec is not used anymore. It is here becaue other addons rely on the same API
 
-        var msgUriSpec = Enigmail.msg.getCurrentMsgUriSpec();
+        let uriSpec = (uri ? uri.spec : null);
 
-        EnigmailCommon.DEBUG_LOG("enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.updateSecurityStatus: msgUriSpec="+msgUriSpec+"\n");
+        EnigmailCommon.DEBUG_LOG("enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.updateSecurityStatus: uri.spec="+uriSpec+"\n");
 
-        if (!uriSpec || (uriSpec == msgUriSpec)) {
-          Enigmail.hdrView.updateHdrIcons(exitCode, statusFlags, keyId, userId, sigDetails, errorMsg, blockSeparation);
+        let msgUriSpec = Enigmail.msg.getCurrentMsgUriSpec();
+
+        let url = {};
+        try{
+          let messenger = Components.classes["@mozilla.org/messenger;1"].getService(Components.interfaces.nsIMessenger);
+          let msgSvc = messenger.messageServiceFromURI(msgUriSpec);
+          msgSvc.GetUrlForUri(msgUriSpec, url, null);
+        }
+        catch (ex) {
+          EnigmailCommon.DEBUG_LOG("enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.updateSecurityStatus: could not determine URL\n");
+          url.value = { spec: "enigmail://invalid/message" };
+        }
+
+        EnigmailCommon.DEBUG_LOG("enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.updateSecurityStatus: url="+url.value.spec+"\n");
+
+        if (!uriSpec || uriSpec.search(/^enigmail:/) == 0 || (uriSpec.indexOf(url.value.spec) == 0 &&
+              uriSpec.substr(url.value.spec.length).search(/([\?&].*)?$/) == 0)) {
+          Enigmail.hdrView.updateHdrIcons(exitCode, statusFlags, keyId, userId, sigDetails,
+                                          errorMsg, blockSeparation, encToDetails,
+                                          null);   // xtraStatus
+        }
+
+        if (uriSpec && uriSpec.search(/^enigmail:message\//) == 0) {
+          // display header for broken MS-Exchange message
+          let ebeb = document.getElementById("enigmailBrokenExchangeBox");
+          ebeb.removeAttribute("collapsed");
         }
 
         return;
@@ -951,5 +1061,5 @@ if (messageHeaderSink) {
     if (!enigmailHeaderSink) {
       this.securityInfo = new EnigMimeHeaderSink(innerSMIMEHeaderSink);
     }
-  }
+  };
 }
